#ifndef DIKT_H
#define DIKT_H

#include "dict.h"
#include "dictpage.h"
#include "dictsettings.h"
#include "dictwidgets.h"

#include <KXmlGuiWindow>
#include <QClipboard>

class QSplitter;
class QStackedWidget;
class QComboBox;
class QLabel;
class KSelectAction;
class DictBrowser;

class Dikt : public KXmlGuiWindow
{
	Q_OBJECT

public:
	static const char *appname;
	static const char *version;
	Dikt(QWidget *parent = 0);
	~Dikt();

public Q_SLOTS:
	void define(const QString &query);
	void match(const QString &query);
	void setHost(const QString &host);

protected Q_SLOTS:
	void define();
	void match();
	void define(const QUrl &link);
	void setDatabaseMenu(const DictList &list);
	void setStrategyMenu(const DictList &list);

	void configure();
	void authentication();
	void introduction();
	void about(const QString &query);
	void info(QAction *action);
	void response(const QString &text);
	void response(const DictResponse &response);
	void error(const QString &text);
	void cancel();

	void newWindow();
	void save();
	void print();
	void preview();

Q_SIGNALS:
	void loading(bool);

private:
	static const int maxQueryLen = 120;
	QComboBox *entry;
	DictBrowser *browser, *matchList;
	DictHistory *backwardMenu, *forwardMenu;
	QMenu *infoMenu;
	KSelectAction *hostMenu;
	DictSelect *strategyMenu;
	DictMenu *databaseMenu;
	DictView *databaseList;
	Dict *dict;

	KSharedConfigPtr config;
	DictSettings *settings;
	QString query;
	DictSearch *search;
	QSplitter *splitter;
	QStackedWidget *sidebar;
	KAction *stop;
	DictStatus *hostStatus, *userStatus;
	DictConf conf;
	DictPage page;

	struct DictTracker {
		bool db, strat, define, match, correct, append;
		int hits, misses, dbcount;
		void clear() { reset();
		define = match = correct = append = false; };
		void reset() { hits = misses = 0; };
		int count() { return hits + misses; };
		void defining(int count) { define = true; dbcount = count; };
		void matching(int count) { match = true; dbcount = count; };
	} tracker;

	void setupActions();
	void setupBrowser();
	void setupDict();
	void setupStatusbar();
	void dicturl(const QUrl &url);

	void init();
	int dbcount() { return databases().size(); };
	QStringList database() { return databaseMenu->current(); };
	QString strategy() { return strategyMenu->current(); };
	QStringList databases() { return (!database().isEmpty()) ? database() : QStringList(Dict::DefaultDatabase); };
	QString strategies() { return (!strategy().isEmpty() && !tracker.correct) ? strategy() : Dict::DefaultStrategy; };

private Q_SLOTS:
	void readConfig();
	void writeConfig();
	QStringList databaseConfig();
	QString strategyConfig();
	void hostConfig(const QString &host);
	void writeHostConfig();
	void readServerConfig();
	void writeServerConfig();
	void readWidthsConfig();
	void writeWidthsConfig();
	void setStyle();

	void loadSettings();
	void setHostStatus(bool connected);
	void setUserStatus(bool authenticated);

	void statusMessage(const QString &message);
	void toggleMenubar();
	void setSidebar(int side);
	void setSidebar(QWidget *widget);
	void toggleDatabaseList();
	void toggleMatchList();
	void enableMatchList(bool enable);
	void nextFrame();
	void yankMode(bool toggle);
	void yankClipboard(QClipboard::Mode);
};

#endif
